#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "odicon.h"
#include "laminar.h"


/* f1(x) = x0^2 + (x1-2)^2 + 2*(x2-5)^2 + 4*x3 */
static double f1(int dim, int x[]) {
    double r = 0;

    assert(dim == 4);  /* check dimension */
    r += x[0] * x[0];
    r += (x[1] - 2) * (x[1] - 2);
    r += 2 * (x[2] - 5) * (x[2] - 5);
    r += 4 * x[3];
    return r;
}


/* f2(x) = ||x+3*e||^2 / 2 */
static double f2(int dim, int x[]) {
    int i;
    double r = 0;

    for (i=0; i<dim; i++) r += (x[i]+3) * (x[i]+3);
    return r/2;
}


static double f3(int dim, int x[]) {
    static int v[] = {
        2,40,90, 1,2,   -1,/* y=(x1+x2),    f += 2y^2+40y+233 */
        50,-2,0, 1,     -1,/* y=(x1),       f += 50y^2-2y */
        3,20,0,  1,2,3, -1,/* y=(x1+x2+x3), f += 3y^2+20y */
        1,1,0,   0,     -1,/* y=(x0),       f += y^2 + y */
        0,0,0,             /* end mark */
    };
    return ilaminar2(dim, x, v);
}


static void test(int dim, double f(int dim, int x[]), 
                 int init[], int lower[], int upper[], double ans) {
    int *x;

    x = ivector_new(dim);
    printf("naive\n");
    ivector_copy(dim, x, init);
    assert( mconv_minimize(dim, f, x, lower, upper) == ans );
    printf("greedy\n");
    ivector_copy(dim, x, init);
    assert( mconv_minimize2(dim, f, x) == ans );
    printf("scaling\n");
    ivector_copy(dim, x, init);
    assert( mconv_minimize_scaling(dim, f, x, lower, upper) == ans );
    printf("--------------------------\n");
    free(x);
}


void mconv2(void) {
    int *init1  = ivector_new(4  /* dim */, 10, 0, 0, 0);
    int *init2  = ivector_new(4  /* dim */, 0, 0, 0, 10);
    int *init3  = ivector_new(10 /* dim */, 10, 20, 30, 40, 50,
                              60, 70, 80, 90, -440);
    int *init4  = ivector_new(10 /* dim */, 100, 200, 300, 400, 500,
                              600, 700, 800, 900, -4400);
    int *lower10 = ivector_new(10 /* dim */,
                               -10000, -10000, -10000, -10000, -10000,
                               -10000, -10000, -10000, -10000, -10000);
    int *upper10 = ivector_new(10 /* dim */,
                               10000,  10000,  10000,  10000,  10000,
                               10000,  10000,  10000,  10000,  10000);

    test(4,  f1, init1, lower10, upper10,  2);
    test(4,  f1, init2, lower10, upper10,  2);
    test(4,  f2, init2, lower10, upper10, 61);
    test(10, f2, init3, lower10, upper10, 80);
    test(10, f2, init4, lower10, upper10, 845);
    test(4,  f3, init1, lower10, upper10,   0);
}

struct laminar_st *laminar;
static double f4(int dim, int x[]) { return ilaminartt(dim, x, laminar); }
static double ff4(int dim, double x[]) { return flaminartt(dim, x, laminar); }

void mconv(const int dim) {
    int i;
    double *fx = fvector_new(dim);
    int *init  = ivector_new(dim);
    int *x     = ivector_new(dim);
    int *lower = ivector_new(dim);
    int *upper = ivector_new(dim);
    int ans, this_ans;

    for (i=0; i<dim; i++) {
        fx[i] = init[i] = irandom(-10*dim, 10*dim);
        lower[i] = -10*dim;
        upper[i] =  10*dim;
    }

    laminar = laminertt_make(dim);
    /*laminar_print(laminar);*/

    lbfgs_call(dim, ff4, fx);
    printf("[lbfgs + sd3]\n");
    for (i=0; i<dim; i++) x[i] = floor(0.5 + fx[i]);
    ans = mgconv_minimize3(dim, f4, x, lower, upper);

    /*
    printf("[lbfgs + sd2]\n");
    for (i=0; i<dim; i++) x[i] = floor(0.5 + fx[i]);
    this_ans = mgconv_minimize2(dim, f4, x);
    if (fcmp(ans, this_ans) != 0) {  printf("error\n");  exit(1);  }
    */

    if (dim <= 180) {
        printf("[scaling]\n");
        ivector_copy(dim, x, init);
        this_ans = mgconv_minimize_scaling(dim, f4, x, lower, upper);
        if (fcmp(ans, this_ans) != 0) {  printf("error\n");  exit(1);  }
    }

    if (dim <= 90) {
        printf("[sd3]\n");
        ivector_copy(dim, x, init);
        this_ans = mgconv_minimize3(dim, f4, x, lower, upper);
        if (fcmp(ans, this_ans) != 0) {  printf("error\n");  exit(1);  }
    }

    /*
    if (dim <= 80) {
        printf("[sd2]\n");
        ivector_copy(dim, x, init);
        this_ans = mgconv_minimize2(dim, f4, x);
        if (fcmp(ans, this_ans) != 0) {  printf("error\n");  exit(1);  }
    }
    */

    if (dim <= 40) {
        printf("[sd]\n");
        ivector_copy(dim, x, init);
        this_ans = mgconv_minimize(dim, f4, x, lower, upper);
        if (fcmp(ans, this_ans) != 0) {  printf("error\n");  exit(1);  }
    }
}
