

#define TRUE		1
#define FALSE		0

#include <float.h>
#define PLUS_INFINITY	DBL_MAX
#define MINUS_INFINITY	DBL_MIN

extern int oracle_call, improve_call, iteration;
extern double cpu_time_limit;
void counter_reset(void);
void counter_print(void);

enum { IFF, WOLFE, IFF_WOLFE };
extern int submodular_type;

int   is_feasible(int dim, double func(int,int*), int x[]);
void status_print(int dim, double func(int,int*), int x[]);

int fcmp(double a, double b);
double int_func(int dim, double double_func(int,double*), int x[]);
double intint_func(int dim, double double_func(int,double*), int x[]);

void *xmalloc(size_t size);

/* constractor of vector */
int    *ivector_new(int dim, ...);
double *fvector_new(int dim, ...);

/* print vector */
void ivector_print(int dim, const int    x[]);
void fvector_print(int dim, const double x[]);

/* copy vector */
int    *ivector_copy(int dim, int    *dst, const int    *src);
double *fvector_copy(int dim, double *dst, const double *src);

/* constractor of matrix */
int    **imatrix_new(int m, int n);
double **fmatrix_new(int m, int n);

int ivector_scan(const int dim, int    *a);
int fvector_scan(const int dim, double *a);
int imatrix_scan(const int dim, int    **aa);
int fmatrix_scan(const int dim, double **aa);

/* [min,max] random */
void mt_initialize(long seed);
int    irandom(int    min, int    max);
double frandom(double min, double max);

/* CPU time */
double cpu_time(void);

/* submodular function minimization */
double sf_minimize(int dim, double func(int,int*), int x[], int direction,
                   int set[]);

/* minimization of M-convex function      */
/* returns the minimum value of func(x)   */
/* INPUT  func  : pointer to the function */
/* INPUT  init[] : the initial solution    */
/* OUTPUT init[] : the minimized solution  */
/* steepest descent algorithm */
double  mconv_minimize(int dim, double func(int dim, int x[]),
                       int init[], int lower[], int upper[]);
double mgconv_minimize(int dim, double func(int dim, int x[]),
                       int init[], int lower[], int upper[]);
/* greedy algorithm (improved steepest descent method) */
double  mconv_minimize2(int dim, double func(int dim, int x[]), int init[]);
double mgconv_minimize2(int dim, double func(int dim, int x[]), int init[]);
/* greedy algorithm (improved steepest descent method) */
double  mconv_minimize3(int dim, double func(int dim, int x[]),
                        int init[], int lower[], int upper[]);
double mgconv_minimize3(int dim, double func(int dim, int x[]),
                        int init[], int lower[], int upper[]);
/* scaling algorithm (with greedy) */
double  mconv_minimize_scaling(int dim, double func(int dim, int x[]),
                               int init[], int lower[], int upper[]);
double mgconv_minimize_scaling(int dim, double func(int dim, int x[]),
                               int init[], int lower[], int upper[]);

/* steepest descent algorithm (experimental) */
double  lconv_minimize_type(int dim, double func(int dim, int x[]), int init[],
			    const int fix_index, const int type);
double lgconv_minimize_type(int dim, double func(int dim, int x[]), int init[],
			    const int type);

/* steepest descent algorithm */
double  lconv_minimize(int dim, double func(int dim, int x[]), int init[],
		       const int fix_index);
double lgconv_minimize(int dim, double func(int dim, int x[]), int init[]);
/* steepest descent scaling algorithm */
double  lconv_minimize_scaling(int dim, double func(int dim, int x[]),
                               int init[], const int fix_index);
double lgconv_minimize_scaling(int dim, double func(int dim, int x[]),
                               int init[]);
/* steepest descent algorithm (with IFF) */
double lgconv_minimize_IFF(int dim, double func(int,int*), int init[]);
/* steepest descent scaling algorithm (with IFF) */
double lgconv_minimize_scaling_IFF(int dim, double func(int,int*), int init[]);
/* steepest descent algorithm (with WOLFE) */
double lgconv_minimize_WOLFE(int dim, double func(int,int*), int init[]);
/* steepest descent scaling algorithm (with WOLFE) */
double lgconv_minimize_scaling_WOLFE(int dim, double func(int,int*),int init[]);

double lbfgs_call(int dim, double func(int dim, double x[]), double init[]);

struct laminar_st;
void mconv_nuopt_call(int dim, struct laminar_st *p, int init[]);
void lgconv_nuopt_call(int dim, int **saa, int **sbb,
                      int *sa, int *sb, int *sd, int *sx);

void mconv(const int dim);
void mconv2(void);
void lconv(const int dim);
void lconv2(const int dim);
void qlconv(const int dim);
void lconv_long(const int dim);
void submodular(const int dim);

void demo_Lseparable(const int dim);
void demo_Mseparable(const int dim);
void demo_procurement(const int dim);
